"""Restore FERC1 plant output tables

Revision ID: 98836fb06355
Revises: 9dfb4295511e
Create Date: 2024-02-21 14:15:08.346846

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '98836fb06355'
down_revision = '9dfb4295511e'
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('out_ferc1__yearly_hydroelectric_plants_sched406',
    sa.Column('report_year', sa.Integer(), nullable=True, comment='Four-digit year in which the data was reported.'),
    sa.Column('utility_id_ferc1', sa.Integer(), nullable=True, comment='PUDL-assigned utility ID, identifying a FERC1 utility. This is an auto-incremented ID and is not expected to be stable from year to year.'),
    sa.Column('utility_id_pudl', sa.Integer(), nullable=True, comment='A manually assigned PUDL utility ID. May not be stable over time.'),
    sa.Column('utility_name_ferc1', sa.Text(), nullable=True, comment='Name of the responding utility, as it is reported in FERC Form 1. For human readability only.'),
    sa.Column('plant_name_ferc1', sa.Text(), nullable=True, comment='Name of the plant, as reported to FERC. This is a freeform string, not guaranteed to be consistent across references to the same plant.'),
    sa.Column('record_id', sa.Text(), nullable=False, comment='Identifier indicating original FERC Form 1 source record. format: {table_name}_{report_year}_{report_prd}_{respondent_id}_{spplmnt_num}_{row_number}. Unique within FERC Form 1 DB tables which are not row-mapped.'),
    sa.Column('asset_retirement_cost', sa.Float(), nullable=True, comment='Asset retirement cost (USD).'),
    sa.Column('avg_num_employees', sa.Float(), nullable=True, comment='The average number of employees assigned to each plant.'),
    sa.Column('capacity_factor', sa.Float(), nullable=True, comment='Fraction of potential generation that was actually reported for a plant part.'),
    sa.Column('capacity_mw', sa.Float(), nullable=True, comment='Total installed (nameplate) capacity, in megawatts.'),
    sa.Column('capex_equipment', sa.Float(), nullable=True, comment='Cost of plant: equipment (USD).'),
    sa.Column('capex_facilities', sa.Float(), nullable=True, comment='Cost of plant: reservoirs, dams, and waterways (USD).'),
    sa.Column('capex_land', sa.Float(), nullable=True, comment='Cost of plant: land and land rights (USD).'),
    sa.Column('capex_per_mw', sa.Float(), nullable=True, comment='Cost of plant per megawatt of installed (nameplate) capacity. Nominal USD.'),
    sa.Column('capex_roads', sa.Float(), nullable=True, comment='Cost of plant: roads, railroads, and bridges (USD).'),
    sa.Column('capex_structures', sa.Float(), nullable=True, comment='Cost of plant: structures and improvements (USD).'),
    sa.Column('capex_total', sa.Float(), nullable=True, comment='Total cost of plant (USD).'),
    sa.Column('construction_type', sa.Enum('conventional', 'outdoor', 'semioutdoor'), nullable=True, comment="Type of plant construction ('outdoor', 'semioutdoor', or 'conventional'). Categorized by PUDL based on our best guess of intended value in FERC1 freeform strings."),
    sa.Column('construction_year', sa.Integer(), nullable=True, comment="Year the plant's oldest still operational unit was built."),
    sa.Column('installation_year', sa.Integer(), nullable=True, comment="Year the plant's most recently built unit was installed."),
    sa.Column('net_capacity_adverse_conditions_mw', sa.Float(), nullable=True, comment='Net plant capability under the least favorable operating conditions, in megawatts.'),
    sa.Column('net_capacity_favorable_conditions_mw', sa.Float(), nullable=True, comment='Net plant capability under the most favorable operating conditions, in megawatts.'),
    sa.Column('net_generation_mwh', sa.Float(), nullable=True, comment='Net electricity generation for the specified period in megawatt-hours (MWh).'),
    sa.Column('opex_dams', sa.Float(), nullable=True, comment='Production expenses: maintenance of reservoirs, dams, and waterways (USD).'),
    sa.Column('opex_electric', sa.Float(), nullable=True, comment='Production expenses: electric expenses (USD).'),
    sa.Column('opex_engineering', sa.Float(), nullable=True, comment='Production expenses: maintenance, supervision, and engineering (USD).'),
    sa.Column('opex_generation_misc', sa.Float(), nullable=True, comment='Production expenses: miscellaneous power generation expenses (USD).'),
    sa.Column('opex_hydraulic', sa.Float(), nullable=True, comment='Production expenses: hydraulic expenses (USD).'),
    sa.Column('opex_misc_plant', sa.Float(), nullable=True, comment='Production expenses: maintenance of miscellaneous hydraulic plant (USD).'),
    sa.Column('opex_operations', sa.Float(), nullable=True, comment='Production expenses: operations, supervision, and engineering (USD).'),
    sa.Column('opex_per_mwh', sa.Float(), nullable=True, comment='Total production expenses (USD per MWh generated).'),
    sa.Column('opex_plant', sa.Float(), nullable=True, comment='Production expenses: maintenance of electric plant (USD).'),
    sa.Column('opex_rents', sa.Float(), nullable=True, comment='Production expenses: rents (USD).'),
    sa.Column('opex_structures', sa.Float(), nullable=True, comment='Production expenses: maintenance of structures (USD).'),
    sa.Column('opex_total', sa.Float(), nullable=True, comment='Total production expenses, excluding fuel (USD).'),
    sa.Column('opex_total_nonfuel', sa.Float(), nullable=True, comment='Total production expenses, excluding fuel (USD).'),
    sa.Column('opex_water_for_power', sa.Float(), nullable=True, comment='Production expenses: water for power (USD).'),
    sa.Column('peak_demand_mw', sa.Float(), nullable=True, comment='Net peak demand for 60 minutes. Note: in some cases peak demand for other time periods may have been reported instead, if hourly peak demand was unavailable.'),
    sa.Column('plant_hours_connected_while_generating', sa.Float(), nullable=True, comment='Hours the plant was connected to load while generating in the report year.'),
    sa.Column('plant_id_pudl', sa.Integer(), nullable=True, comment='A manually assigned PUDL plant ID. May not be constant over time.'),
    sa.Column('plant_type', sa.Text(), nullable=True, comment='Type of plant.'),
    sa.Column('project_num', sa.Integer(), nullable=True, comment='FERC Licensed Project Number.'),
    sa.ForeignKeyConstraint(['plant_id_pudl'], ['core_pudl__entity_plants_pudl.plant_id_pudl'], name=op.f('fk_out_ferc1__yearly_hydroelectric_plants_sched406_plant_id_pudl_core_pudl__entity_plants_pudl')),
    sa.ForeignKeyConstraint(['utility_id_ferc1', 'plant_name_ferc1'], ['core_pudl__assn_ferc1_pudl_plants.utility_id_ferc1', 'core_pudl__assn_ferc1_pudl_plants.plant_name_ferc1'], name=op.f('fk_out_ferc1__yearly_hydroelectric_plants_sched406_utility_id_ferc1_core_pudl__assn_ferc1_pudl_plants')),
    sa.ForeignKeyConstraint(['utility_id_pudl'], ['core_pudl__entity_utilities_pudl.utility_id_pudl'], name=op.f('fk_out_ferc1__yearly_hydroelectric_plants_sched406_utility_id_pudl_core_pudl__entity_utilities_pudl')),
    sa.PrimaryKeyConstraint('record_id', name=op.f('pk_out_ferc1__yearly_hydroelectric_plants_sched406'))
    )
    op.create_table('out_ferc1__yearly_pumped_storage_plants_sched408',
    sa.Column('report_year', sa.Integer(), nullable=True, comment='Four-digit year in which the data was reported.'),
    sa.Column('utility_id_ferc1', sa.Integer(), nullable=True, comment='PUDL-assigned utility ID, identifying a FERC1 utility. This is an auto-incremented ID and is not expected to be stable from year to year.'),
    sa.Column('utility_id_pudl', sa.Integer(), nullable=True, comment='A manually assigned PUDL utility ID. May not be stable over time.'),
    sa.Column('utility_name_ferc1', sa.Text(), nullable=True, comment='Name of the responding utility, as it is reported in FERC Form 1. For human readability only.'),
    sa.Column('plant_name_ferc1', sa.Text(), nullable=True, comment='Name of the plant, as reported to FERC. This is a freeform string, not guaranteed to be consistent across references to the same plant.'),
    sa.Column('record_id', sa.Text(), nullable=False, comment='Identifier indicating original FERC Form 1 source record. format: {table_name}_{report_year}_{report_prd}_{respondent_id}_{spplmnt_num}_{row_number}. Unique within FERC Form 1 DB tables which are not row-mapped.'),
    sa.Column('asset_retirement_cost', sa.Float(), nullable=True, comment='Asset retirement cost (USD).'),
    sa.Column('avg_num_employees', sa.Float(), nullable=True, comment='The average number of employees assigned to each plant.'),
    sa.Column('capacity_factor', sa.Float(), nullable=True, comment='Fraction of potential generation that was actually reported for a plant part.'),
    sa.Column('capacity_mw', sa.Float(), nullable=True, comment='Total installed (nameplate) capacity, in megawatts.'),
    sa.Column('capex_equipment_electric', sa.Float(), nullable=True, comment='Cost of plant: accessory electric equipment (USD).'),
    sa.Column('capex_equipment_misc', sa.Float(), nullable=True, comment='Cost of plant: miscellaneous power plant equipment (USD).'),
    sa.Column('capex_facilities', sa.Float(), nullable=True, comment='Cost of plant: reservoirs, dams, and waterways (USD).'),
    sa.Column('capex_land', sa.Float(), nullable=True, comment='Cost of plant: land and land rights (USD).'),
    sa.Column('capex_per_mw', sa.Float(), nullable=True, comment='Cost of plant per megawatt of installed (nameplate) capacity. Nominal USD.'),
    sa.Column('capex_roads', sa.Float(), nullable=True, comment='Cost of plant: roads, railroads, and bridges (USD).'),
    sa.Column('capex_structures', sa.Float(), nullable=True, comment='Cost of plant: structures and improvements (USD).'),
    sa.Column('capex_total', sa.Float(), nullable=True, comment='Total cost of plant (USD).'),
    sa.Column('capex_wheels_turbines_generators', sa.Float(), nullable=True, comment='Cost of plant: water wheels, turbines, and generators (USD).'),
    sa.Column('construction_type', sa.Enum('conventional', 'outdoor', 'semioutdoor'), nullable=True, comment="Type of plant construction ('outdoor', 'semioutdoor', or 'conventional'). Categorized by PUDL based on our best guess of intended value in FERC1 freeform strings."),
    sa.Column('construction_year', sa.Integer(), nullable=True, comment="Year the plant's oldest still operational unit was built."),
    sa.Column('energy_used_for_pumping_mwh', sa.Float(), nullable=True, comment='Energy used for pumping, in megawatt-hours.'),
    sa.Column('installation_year', sa.Integer(), nullable=True, comment="Year the plant's most recently built unit was installed."),
    sa.Column('net_generation_mwh', sa.Float(), nullable=True, comment='Net electricity generation for the specified period in megawatt-hours (MWh).'),
    sa.Column('net_load_mwh', sa.Float(), nullable=True, comment='Net output for load (net generation - energy used for pumping) in megawatt-hours.'),
    sa.Column('opex_dams', sa.Float(), nullable=True, comment='Production expenses: maintenance of reservoirs, dams, and waterways (USD).'),
    sa.Column('opex_electric', sa.Float(), nullable=True, comment='Production expenses: electric expenses (USD).'),
    sa.Column('opex_engineering', sa.Float(), nullable=True, comment='Production expenses: maintenance, supervision, and engineering (USD).'),
    sa.Column('opex_generation_misc', sa.Float(), nullable=True, comment='Production expenses: miscellaneous power generation expenses (USD).'),
    sa.Column('opex_misc_plant', sa.Float(), nullable=True, comment='Production expenses: maintenance of miscellaneous hydraulic plant (USD).'),
    sa.Column('opex_operations', sa.Float(), nullable=True, comment='Production expenses: operations, supervision, and engineering (USD).'),
    sa.Column('opex_per_mwh', sa.Float(), nullable=True, comment='Total production expenses (USD per MWh generated).'),
    sa.Column('opex_plant', sa.Float(), nullable=True, comment='Production expenses: maintenance of electric plant (USD).'),
    sa.Column('opex_production_before_pumping', sa.Float(), nullable=True, comment='Total production expenses before pumping (USD).'),
    sa.Column('opex_pumped_storage', sa.Float(), nullable=True, comment='Production expenses: pumped storage (USD).'),
    sa.Column('opex_pumping', sa.Float(), nullable=True, comment='Production expenses: We are here to PUMP YOU UP! (USD).'),
    sa.Column('opex_rents', sa.Float(), nullable=True, comment='Production expenses: rents (USD).'),
    sa.Column('opex_structures', sa.Float(), nullable=True, comment='Production expenses: maintenance of structures (USD).'),
    sa.Column('opex_total', sa.Float(), nullable=True, comment='Total production expenses, excluding fuel (USD).'),
    sa.Column('opex_total_nonfuel', sa.Float(), nullable=True, comment='Total production expenses, excluding fuel (USD).'),
    sa.Column('opex_water_for_power', sa.Float(), nullable=True, comment='Production expenses: water for power (USD).'),
    sa.Column('peak_demand_mw', sa.Float(), nullable=True, comment='Net peak demand for 60 minutes. Note: in some cases peak demand for other time periods may have been reported instead, if hourly peak demand was unavailable.'),
    sa.Column('plant_capability_mw', sa.Float(), nullable=True, comment='Net plant capability in megawatts.'),
    sa.Column('plant_hours_connected_while_generating', sa.Float(), nullable=True, comment='Hours the plant was connected to load while generating in the report year.'),
    sa.Column('plant_id_pudl', sa.Integer(), nullable=True, comment='A manually assigned PUDL plant ID. May not be constant over time.'),
    sa.Column('project_num', sa.Integer(), nullable=True, comment='FERC Licensed Project Number.'),
    sa.ForeignKeyConstraint(['plant_id_pudl'], ['core_pudl__entity_plants_pudl.plant_id_pudl'], name=op.f('fk_out_ferc1__yearly_pumped_storage_plants_sched408_plant_id_pudl_core_pudl__entity_plants_pudl')),
    sa.ForeignKeyConstraint(['utility_id_ferc1', 'plant_name_ferc1'], ['core_pudl__assn_ferc1_pudl_plants.utility_id_ferc1', 'core_pudl__assn_ferc1_pudl_plants.plant_name_ferc1'], name=op.f('fk_out_ferc1__yearly_pumped_storage_plants_sched408_utility_id_ferc1_core_pudl__assn_ferc1_pudl_plants')),
    sa.ForeignKeyConstraint(['utility_id_pudl'], ['core_pudl__entity_utilities_pudl.utility_id_pudl'], name=op.f('fk_out_ferc1__yearly_pumped_storage_plants_sched408_utility_id_pudl_core_pudl__entity_utilities_pudl')),
    sa.PrimaryKeyConstraint('record_id', name=op.f('pk_out_ferc1__yearly_pumped_storage_plants_sched408'))
    )
    op.create_table('out_ferc1__yearly_small_plants_sched410',
    sa.Column('report_year', sa.Integer(), nullable=True, comment='Four-digit year in which the data was reported.'),
    sa.Column('utility_id_ferc1', sa.Integer(), nullable=True, comment='PUDL-assigned utility ID, identifying a FERC1 utility. This is an auto-incremented ID and is not expected to be stable from year to year.'),
    sa.Column('utility_id_pudl', sa.Integer(), nullable=True, comment='A manually assigned PUDL utility ID. May not be stable over time.'),
    sa.Column('utility_name_ferc1', sa.Text(), nullable=True, comment='Name of the responding utility, as it is reported in FERC Form 1. For human readability only.'),
    sa.Column('plant_id_pudl', sa.Integer(), nullable=True, comment='A manually assigned PUDL plant ID. May not be constant over time.'),
    sa.Column('plant_name_ferc1', sa.Text(), nullable=True, comment='Name of the plant, as reported to FERC. This is a freeform string, not guaranteed to be consistent across references to the same plant.'),
    sa.Column('record_id', sa.Text(), nullable=False, comment='Identifier indicating original FERC Form 1 source record. format: {table_name}_{report_year}_{report_prd}_{respondent_id}_{spplmnt_num}_{row_number}. Unique within FERC Form 1 DB tables which are not row-mapped.'),
    sa.Column('capacity_mw', sa.Float(), nullable=True, comment='Total installed (nameplate) capacity, in megawatts.'),
    sa.Column('capex_per_mw', sa.Float(), nullable=True, comment='Cost of plant per megawatt of installed (nameplate) capacity. Nominal USD.'),
    sa.Column('capex_total', sa.Float(), nullable=True, comment='Total cost of plant (USD).'),
    sa.Column('construction_year', sa.Integer(), nullable=True, comment="Year the plant's oldest still operational unit was built."),
    sa.Column('fuel_cost_per_mmbtu', sa.Float(), nullable=True, comment='Average fuel cost per mmBTU of heat content in nominal USD.'),
    sa.Column('fuel_type', sa.Text(), nullable=True, comment='Type of fuel.'),
    sa.Column('license_id_ferc1', sa.Integer(), nullable=True, comment='FERC issued operating license ID for the facility, if available. This value is extracted from the original plant name where possible.'),
    sa.Column('net_generation_mwh', sa.Float(), nullable=True, comment='Net electricity generation for the specified period in megawatt-hours (MWh).'),
    sa.Column('opex_fuel', sa.Float(), nullable=True, comment='Production expenses: fuel (USD).'),
    sa.Column('opex_maintenance', sa.Float(), nullable=True, comment='Production expenses: Maintenance (USD).'),
    sa.Column('opex_operations', sa.Float(), nullable=True, comment='Production expenses: operations, supervision, and engineering (USD).'),
    sa.Column('opex_total', sa.Float(), nullable=True, comment='Total production expenses, excluding fuel (USD).'),
    sa.Column('opex_total_nonfuel', sa.Float(), nullable=True, comment='Total production expenses, excluding fuel (USD).'),
    sa.Column('peak_demand_mw', sa.Float(), nullable=True, comment='Net peak demand for 60 minutes. Note: in some cases peak demand for other time periods may have been reported instead, if hourly peak demand was unavailable.'),
    sa.Column('plant_type', sa.Text(), nullable=True, comment='Type of plant.'),
    sa.ForeignKeyConstraint(['plant_id_pudl'], ['core_pudl__entity_plants_pudl.plant_id_pudl'], name=op.f('fk_out_ferc1__yearly_small_plants_sched410_plant_id_pudl_core_pudl__entity_plants_pudl')),
    sa.ForeignKeyConstraint(['utility_id_ferc1', 'plant_name_ferc1'], ['core_pudl__assn_ferc1_pudl_plants.utility_id_ferc1', 'core_pudl__assn_ferc1_pudl_plants.plant_name_ferc1'], name=op.f('fk_out_ferc1__yearly_small_plants_sched410_utility_id_ferc1_core_pudl__assn_ferc1_pudl_plants')),
    sa.ForeignKeyConstraint(['utility_id_pudl'], ['core_pudl__entity_utilities_pudl.utility_id_pudl'], name=op.f('fk_out_ferc1__yearly_small_plants_sched410_utility_id_pudl_core_pudl__entity_utilities_pudl')),
    sa.PrimaryKeyConstraint('record_id', name=op.f('pk_out_ferc1__yearly_small_plants_sched410'))
    )
    op.create_table('out_ferc1__yearly_steam_plants_sched402',
    sa.Column('report_year', sa.Integer(), nullable=True, comment='Four-digit year in which the data was reported.'),
    sa.Column('utility_id_ferc1', sa.Integer(), nullable=True, comment='PUDL-assigned utility ID, identifying a FERC1 utility. This is an auto-incremented ID and is not expected to be stable from year to year.'),
    sa.Column('utility_id_pudl', sa.Integer(), nullable=True, comment='A manually assigned PUDL utility ID. May not be stable over time.'),
    sa.Column('utility_name_ferc1', sa.Text(), nullable=True, comment='Name of the responding utility, as it is reported in FERC Form 1. For human readability only.'),
    sa.Column('plant_id_pudl', sa.Integer(), nullable=True, comment='A manually assigned PUDL plant ID. May not be constant over time.'),
    sa.Column('plant_id_ferc1', sa.Integer(), nullable=True, comment='Algorithmically assigned PUDL FERC Plant ID. WARNING: NOT STABLE BETWEEN PUDL DB INITIALIZATIONS.'),
    sa.Column('plant_name_ferc1', sa.Text(), nullable=True, comment='Name of the plant, as reported to FERC. This is a freeform string, not guaranteed to be consistent across references to the same plant.'),
    sa.Column('asset_retirement_cost', sa.Float(), nullable=True, comment='Asset retirement cost (USD).'),
    sa.Column('avg_num_employees', sa.Float(), nullable=True, comment='The average number of employees assigned to each plant.'),
    sa.Column('capacity_factor', sa.Float(), nullable=True, comment='Fraction of potential generation that was actually reported for a plant part.'),
    sa.Column('capacity_mw', sa.Float(), nullable=True, comment='Total installed (nameplate) capacity, in megawatts.'),
    sa.Column('capex_annual_addition', sa.Float(), nullable=True, comment='Annual capital addition into `capex_total`.'),
    sa.Column('capex_annual_addition_rolling', sa.Float(), nullable=True, comment='Year-to-date capital addition into `capex_total`.'),
    sa.Column('capex_annual_per_kw', sa.Float(), nullable=True, comment='Annual capital addition into `capex_total` per kw.'),
    sa.Column('capex_annual_per_mw', sa.Float(), nullable=True, comment='Annual capital addition into `capex_total` per MW.'),
    sa.Column('capex_annual_per_mw_rolling', sa.Float(), nullable=True, comment='Year-to-date capital addition into `capex_total` per MW.'),
    sa.Column('capex_annual_per_mwh', sa.Float(), nullable=True, comment='Annual capital addition into `capex_total` per MWh.'),
    sa.Column('capex_annual_per_mwh_rolling', sa.Float(), nullable=True, comment='Year-to-date capital addition into `capex_total` per MWh.'),
    sa.Column('capex_equipment', sa.Float(), nullable=True, comment='Cost of plant: equipment (USD).'),
    sa.Column('capex_land', sa.Float(), nullable=True, comment='Cost of plant: land and land rights (USD).'),
    sa.Column('capex_per_mw', sa.Float(), nullable=True, comment='Cost of plant per megawatt of installed (nameplate) capacity. Nominal USD.'),
    sa.Column('capex_structures', sa.Float(), nullable=True, comment='Cost of plant: structures and improvements (USD).'),
    sa.Column('capex_total', sa.Float(), nullable=True, comment='Total cost of plant (USD).'),
    sa.Column('capex_wo_retirement_total', sa.Float(), nullable=True, comment='Total cost of plant (USD) without retirements.'),
    sa.Column('construction_type', sa.Enum('conventional', 'outdoor', 'semioutdoor'), nullable=True, comment="Type of plant construction ('outdoor', 'semioutdoor', or 'conventional'). Categorized by PUDL based on our best guess of intended value in FERC1 freeform strings."),
    sa.Column('construction_year', sa.Integer(), nullable=True, comment="Year the plant's oldest still operational unit was built."),
    sa.Column('installation_year', sa.Integer(), nullable=True, comment="Year the plant's most recently built unit was installed."),
    sa.Column('net_generation_mwh', sa.Float(), nullable=True, comment='Net electricity generation for the specified period in megawatt-hours (MWh).'),
    sa.Column('not_water_limited_capacity_mw', sa.Float(), nullable=True, comment='Plant capacity in MW when not limited by condenser water.'),
    sa.Column('opex_allowances', sa.Float(), nullable=True, comment='Allowances.'),
    sa.Column('opex_boiler', sa.Float(), nullable=True, comment='Maintenance of boiler (or reactor) plant.'),
    sa.Column('opex_coolants', sa.Float(), nullable=True, comment='Cost of coolants and water (nuclear plants only)'),
    sa.Column('opex_electric', sa.Float(), nullable=True, comment='Production expenses: electric expenses (USD).'),
    sa.Column('opex_engineering', sa.Float(), nullable=True, comment='Production expenses: maintenance, supervision, and engineering (USD).'),
    sa.Column('opex_fuel', sa.Float(), nullable=True, comment='Production expenses: fuel (USD).'),
    sa.Column('opex_fuel_per_mwh', sa.Float(), nullable=True, comment='Production expenses: fuel (USD) per megawatt-hour (Mwh).'),
    sa.Column('opex_misc_power', sa.Float(), nullable=True, comment='Miscellaneous steam (or nuclear) expenses.'),
    sa.Column('opex_misc_steam', sa.Float(), nullable=True, comment='Maintenance of miscellaneous steam (or nuclear) plant.'),
    sa.Column('opex_nonfuel_per_mwh', sa.Float(), nullable=True, comment='Investments in non-fuel production expenses per Mwh.'),
    sa.Column('opex_operations', sa.Float(), nullable=True, comment='Production expenses: operations, supervision, and engineering (USD).'),
    sa.Column('opex_per_mwh', sa.Float(), nullable=True, comment='Total production expenses (USD per MWh generated).'),
    sa.Column('opex_plants', sa.Float(), nullable=True, comment='Maintenance of electrical plant.'),
    sa.Column('opex_production_total', sa.Float(), nullable=True, comment='Total operating expenses.'),
    sa.Column('opex_rents', sa.Float(), nullable=True, comment='Production expenses: rents (USD).'),
    sa.Column('opex_steam', sa.Float(), nullable=True, comment='Steam expenses.'),
    sa.Column('opex_steam_other', sa.Float(), nullable=True, comment='Steam from other sources.'),
    sa.Column('opex_structures', sa.Float(), nullable=True, comment='Production expenses: maintenance of structures (USD).'),
    sa.Column('opex_total_nonfuel', sa.Float(), nullable=True, comment='Total production expenses, excluding fuel (USD).'),
    sa.Column('opex_transfer', sa.Float(), nullable=True, comment='Steam transferred (Credit).'),
    sa.Column('peak_demand_mw', sa.Float(), nullable=True, comment='Net peak demand for 60 minutes. Note: in some cases peak demand for other time periods may have been reported instead, if hourly peak demand was unavailable.'),
    sa.Column('plant_capability_mw', sa.Float(), nullable=True, comment='Net plant capability in megawatts.'),
    sa.Column('plant_hours_connected_while_generating', sa.Float(), nullable=True, comment='Hours the plant was connected to load while generating in the report year.'),
    sa.Column('plant_type', sa.Text(), nullable=True, comment='Type of plant.'),
    sa.Column('record_id', sa.Text(), nullable=False, comment='Identifier indicating original FERC Form 1 source record. format: {table_name}_{report_year}_{report_prd}_{respondent_id}_{spplmnt_num}_{row_number}. Unique within FERC Form 1 DB tables which are not row-mapped.'),
    sa.Column('water_limited_capacity_mw', sa.Float(), nullable=True, comment='Plant capacity in MW when limited by condenser water.'),
    sa.ForeignKeyConstraint(['plant_id_pudl'], ['core_pudl__entity_plants_pudl.plant_id_pudl'], name=op.f('fk_out_ferc1__yearly_steam_plants_sched402_plant_id_pudl_core_pudl__entity_plants_pudl')),
    sa.ForeignKeyConstraint(['utility_id_ferc1', 'plant_name_ferc1'], ['core_pudl__assn_ferc1_pudl_plants.utility_id_ferc1', 'core_pudl__assn_ferc1_pudl_plants.plant_name_ferc1'], name=op.f('fk_out_ferc1__yearly_steam_plants_sched402_utility_id_ferc1_core_pudl__assn_ferc1_pudl_plants')),
    sa.ForeignKeyConstraint(['utility_id_pudl'], ['core_pudl__entity_utilities_pudl.utility_id_pudl'], name=op.f('fk_out_ferc1__yearly_steam_plants_sched402_utility_id_pudl_core_pudl__entity_utilities_pudl')),
    sa.PrimaryKeyConstraint('record_id', name=op.f('pk_out_ferc1__yearly_steam_plants_sched402'))
    )
    op.drop_table('_out_ferc1__yearly_pumped_storage_plants_sched408')
    op.drop_table('_out_ferc1__yearly_small_plants_sched410')
    op.drop_table('_out_ferc1__yearly_hydroelectric_plants_sched406')
    op.drop_table('_out_ferc1__yearly_steam_plants_sched402')
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('_out_ferc1__yearly_steam_plants_sched402',
    sa.Column('report_year', sa.INTEGER(), nullable=True),
    sa.Column('utility_id_ferc1', sa.INTEGER(), nullable=True),
    sa.Column('utility_id_pudl', sa.INTEGER(), nullable=True),
    sa.Column('utility_name_ferc1', sa.TEXT(), nullable=True),
    sa.Column('plant_id_pudl', sa.INTEGER(), nullable=True),
    sa.Column('plant_id_ferc1', sa.INTEGER(), nullable=True),
    sa.Column('plant_name_ferc1', sa.TEXT(), nullable=True),
    sa.Column('asset_retirement_cost', sa.FLOAT(), nullable=True),
    sa.Column('avg_num_employees', sa.FLOAT(), nullable=True),
    sa.Column('capacity_factor', sa.FLOAT(), nullable=True),
    sa.Column('capacity_mw', sa.FLOAT(), nullable=True),
    sa.Column('capex_annual_addition', sa.FLOAT(), nullable=True),
    sa.Column('capex_annual_addition_rolling', sa.FLOAT(), nullable=True),
    sa.Column('capex_annual_per_kw', sa.FLOAT(), nullable=True),
    sa.Column('capex_annual_per_mw', sa.FLOAT(), nullable=True),
    sa.Column('capex_annual_per_mw_rolling', sa.FLOAT(), nullable=True),
    sa.Column('capex_annual_per_mwh', sa.FLOAT(), nullable=True),
    sa.Column('capex_annual_per_mwh_rolling', sa.FLOAT(), nullable=True),
    sa.Column('capex_equipment', sa.FLOAT(), nullable=True),
    sa.Column('capex_land', sa.FLOAT(), nullable=True),
    sa.Column('capex_per_mw', sa.FLOAT(), nullable=True),
    sa.Column('capex_structures', sa.FLOAT(), nullable=True),
    sa.Column('capex_total', sa.FLOAT(), nullable=True),
    sa.Column('capex_wo_retirement_total', sa.FLOAT(), nullable=True),
    sa.Column('construction_type', sa.VARCHAR(length=12), nullable=True),
    sa.Column('construction_year', sa.INTEGER(), nullable=True),
    sa.Column('installation_year', sa.INTEGER(), nullable=True),
    sa.Column('net_generation_mwh', sa.FLOAT(), nullable=True),
    sa.Column('not_water_limited_capacity_mw', sa.FLOAT(), nullable=True),
    sa.Column('opex_allowances', sa.FLOAT(), nullable=True),
    sa.Column('opex_boiler', sa.FLOAT(), nullable=True),
    sa.Column('opex_coolants', sa.FLOAT(), nullable=True),
    sa.Column('opex_electric', sa.FLOAT(), nullable=True),
    sa.Column('opex_engineering', sa.FLOAT(), nullable=True),
    sa.Column('opex_fuel', sa.FLOAT(), nullable=True),
    sa.Column('opex_fuel_per_mwh', sa.FLOAT(), nullable=True),
    sa.Column('opex_misc_power', sa.FLOAT(), nullable=True),
    sa.Column('opex_misc_steam', sa.FLOAT(), nullable=True),
    sa.Column('opex_nonfuel_per_mwh', sa.FLOAT(), nullable=True),
    sa.Column('opex_operations', sa.FLOAT(), nullable=True),
    sa.Column('opex_per_mwh', sa.FLOAT(), nullable=True),
    sa.Column('opex_plants', sa.FLOAT(), nullable=True),
    sa.Column('opex_production_total', sa.FLOAT(), nullable=True),
    sa.Column('opex_rents', sa.FLOAT(), nullable=True),
    sa.Column('opex_steam', sa.FLOAT(), nullable=True),
    sa.Column('opex_steam_other', sa.FLOAT(), nullable=True),
    sa.Column('opex_structures', sa.FLOAT(), nullable=True),
    sa.Column('opex_total_nonfuel', sa.FLOAT(), nullable=True),
    sa.Column('opex_transfer', sa.FLOAT(), nullable=True),
    sa.Column('peak_demand_mw', sa.FLOAT(), nullable=True),
    sa.Column('plant_capability_mw', sa.FLOAT(), nullable=True),
    sa.Column('plant_hours_connected_while_generating', sa.FLOAT(), nullable=True),
    sa.Column('plant_type', sa.TEXT(), nullable=True),
    sa.Column('record_id', sa.TEXT(), nullable=False),
    sa.Column('water_limited_capacity_mw', sa.FLOAT(), nullable=True),
    sa.ForeignKeyConstraint(['plant_id_pudl'], ['core_pudl__entity_plants_pudl.plant_id_pudl'], name='fk__out_ferc1__yearly_steam_plants_sched402_plant_id_pudl_core_pudl__entity_plants_pudl'),
    sa.ForeignKeyConstraint(['utility_id_ferc1', 'plant_name_ferc1'], ['core_pudl__assn_ferc1_pudl_plants.utility_id_ferc1', 'core_pudl__assn_ferc1_pudl_plants.plant_name_ferc1'], name='fk__out_ferc1__yearly_steam_plants_sched402_utility_id_ferc1_core_pudl__assn_ferc1_pudl_plants'),
    sa.ForeignKeyConstraint(['utility_id_pudl'], ['core_pudl__entity_utilities_pudl.utility_id_pudl'], name='fk__out_ferc1__yearly_steam_plants_sched402_utility_id_pudl_core_pudl__entity_utilities_pudl'),
    sa.PrimaryKeyConstraint('record_id', name='pk__out_ferc1__yearly_steam_plants_sched402')
    )
    op.create_table('_out_ferc1__yearly_hydroelectric_plants_sched406',
    sa.Column('report_year', sa.INTEGER(), nullable=True),
    sa.Column('utility_id_ferc1', sa.INTEGER(), nullable=True),
    sa.Column('utility_id_pudl', sa.INTEGER(), nullable=True),
    sa.Column('utility_name_ferc1', sa.TEXT(), nullable=True),
    sa.Column('plant_name_ferc1', sa.TEXT(), nullable=True),
    sa.Column('record_id', sa.TEXT(), nullable=False),
    sa.Column('asset_retirement_cost', sa.FLOAT(), nullable=True),
    sa.Column('avg_num_employees', sa.FLOAT(), nullable=True),
    sa.Column('capacity_factor', sa.FLOAT(), nullable=True),
    sa.Column('capacity_mw', sa.FLOAT(), nullable=True),
    sa.Column('capex_equipment', sa.FLOAT(), nullable=True),
    sa.Column('capex_facilities', sa.FLOAT(), nullable=True),
    sa.Column('capex_land', sa.FLOAT(), nullable=True),
    sa.Column('capex_per_mw', sa.FLOAT(), nullable=True),
    sa.Column('capex_roads', sa.FLOAT(), nullable=True),
    sa.Column('capex_structures', sa.FLOAT(), nullable=True),
    sa.Column('capex_total', sa.FLOAT(), nullable=True),
    sa.Column('construction_type', sa.VARCHAR(length=12), nullable=True),
    sa.Column('construction_year', sa.INTEGER(), nullable=True),
    sa.Column('installation_year', sa.INTEGER(), nullable=True),
    sa.Column('net_capacity_adverse_conditions_mw', sa.FLOAT(), nullable=True),
    sa.Column('net_capacity_favorable_conditions_mw', sa.FLOAT(), nullable=True),
    sa.Column('net_generation_mwh', sa.FLOAT(), nullable=True),
    sa.Column('opex_dams', sa.FLOAT(), nullable=True),
    sa.Column('opex_electric', sa.FLOAT(), nullable=True),
    sa.Column('opex_engineering', sa.FLOAT(), nullable=True),
    sa.Column('opex_generation_misc', sa.FLOAT(), nullable=True),
    sa.Column('opex_hydraulic', sa.FLOAT(), nullable=True),
    sa.Column('opex_misc_plant', sa.FLOAT(), nullable=True),
    sa.Column('opex_operations', sa.FLOAT(), nullable=True),
    sa.Column('opex_per_mwh', sa.FLOAT(), nullable=True),
    sa.Column('opex_plant', sa.FLOAT(), nullable=True),
    sa.Column('opex_rents', sa.FLOAT(), nullable=True),
    sa.Column('opex_structures', sa.FLOAT(), nullable=True),
    sa.Column('opex_total', sa.FLOAT(), nullable=True),
    sa.Column('opex_total_nonfuel', sa.FLOAT(), nullable=True),
    sa.Column('opex_water_for_power', sa.FLOAT(), nullable=True),
    sa.Column('peak_demand_mw', sa.FLOAT(), nullable=True),
    sa.Column('plant_hours_connected_while_generating', sa.FLOAT(), nullable=True),
    sa.Column('plant_id_pudl', sa.INTEGER(), nullable=True),
    sa.Column('plant_type', sa.TEXT(), nullable=True),
    sa.Column('project_num', sa.INTEGER(), nullable=True),
    sa.ForeignKeyConstraint(['plant_id_pudl'], ['core_pudl__entity_plants_pudl.plant_id_pudl'], name='fk__out_ferc1__yearly_hydroelectric_plants_sched406_plant_id_pudl_core_pudl__entity_plants_pudl'),
    sa.ForeignKeyConstraint(['utility_id_ferc1', 'plant_name_ferc1'], ['core_pudl__assn_ferc1_pudl_plants.utility_id_ferc1', 'core_pudl__assn_ferc1_pudl_plants.plant_name_ferc1'], name='fk__out_ferc1__yearly_hydroelectric_plants_sched406_utility_id_ferc1_core_pudl__assn_ferc1_pudl_plants'),
    sa.ForeignKeyConstraint(['utility_id_pudl'], ['core_pudl__entity_utilities_pudl.utility_id_pudl'], name='fk__out_ferc1__yearly_hydroelectric_plants_sched406_utility_id_pudl_core_pudl__entity_utilities_pudl'),
    sa.PrimaryKeyConstraint('record_id', name='pk__out_ferc1__yearly_hydroelectric_plants_sched406')
    )
    op.create_table('_out_ferc1__yearly_small_plants_sched410',
    sa.Column('report_year', sa.INTEGER(), nullable=True),
    sa.Column('utility_id_ferc1', sa.INTEGER(), nullable=True),
    sa.Column('utility_id_pudl', sa.INTEGER(), nullable=True),
    sa.Column('utility_name_ferc1', sa.TEXT(), nullable=True),
    sa.Column('plant_id_pudl', sa.INTEGER(), nullable=True),
    sa.Column('plant_name_ferc1', sa.TEXT(), nullable=True),
    sa.Column('record_id', sa.TEXT(), nullable=False),
    sa.Column('capacity_mw', sa.FLOAT(), nullable=True),
    sa.Column('capex_per_mw', sa.FLOAT(), nullable=True),
    sa.Column('capex_total', sa.FLOAT(), nullable=True),
    sa.Column('construction_year', sa.INTEGER(), nullable=True),
    sa.Column('fuel_cost_per_mmbtu', sa.FLOAT(), nullable=True),
    sa.Column('fuel_type', sa.TEXT(), nullable=True),
    sa.Column('license_id_ferc1', sa.INTEGER(), nullable=True),
    sa.Column('net_generation_mwh', sa.FLOAT(), nullable=True),
    sa.Column('opex_fuel', sa.FLOAT(), nullable=True),
    sa.Column('opex_maintenance', sa.FLOAT(), nullable=True),
    sa.Column('opex_operations', sa.FLOAT(), nullable=True),
    sa.Column('opex_total', sa.FLOAT(), nullable=True),
    sa.Column('opex_total_nonfuel', sa.FLOAT(), nullable=True),
    sa.Column('peak_demand_mw', sa.FLOAT(), nullable=True),
    sa.Column('plant_type', sa.TEXT(), nullable=True),
    sa.ForeignKeyConstraint(['plant_id_pudl'], ['core_pudl__entity_plants_pudl.plant_id_pudl'], name='fk__out_ferc1__yearly_small_plants_sched410_plant_id_pudl_core_pudl__entity_plants_pudl'),
    sa.ForeignKeyConstraint(['utility_id_ferc1', 'plant_name_ferc1'], ['core_pudl__assn_ferc1_pudl_plants.utility_id_ferc1', 'core_pudl__assn_ferc1_pudl_plants.plant_name_ferc1'], name='fk__out_ferc1__yearly_small_plants_sched410_utility_id_ferc1_core_pudl__assn_ferc1_pudl_plants'),
    sa.ForeignKeyConstraint(['utility_id_pudl'], ['core_pudl__entity_utilities_pudl.utility_id_pudl'], name='fk__out_ferc1__yearly_small_plants_sched410_utility_id_pudl_core_pudl__entity_utilities_pudl'),
    sa.PrimaryKeyConstraint('record_id', name='pk__out_ferc1__yearly_small_plants_sched410')
    )
    op.create_table('_out_ferc1__yearly_pumped_storage_plants_sched408',
    sa.Column('report_year', sa.INTEGER(), nullable=True),
    sa.Column('utility_id_ferc1', sa.INTEGER(), nullable=True),
    sa.Column('utility_id_pudl', sa.INTEGER(), nullable=True),
    sa.Column('utility_name_ferc1', sa.TEXT(), nullable=True),
    sa.Column('plant_name_ferc1', sa.TEXT(), nullable=True),
    sa.Column('record_id', sa.TEXT(), nullable=False),
    sa.Column('asset_retirement_cost', sa.FLOAT(), nullable=True),
    sa.Column('avg_num_employees', sa.FLOAT(), nullable=True),
    sa.Column('capacity_factor', sa.FLOAT(), nullable=True),
    sa.Column('capacity_mw', sa.FLOAT(), nullable=True),
    sa.Column('capex_equipment_electric', sa.FLOAT(), nullable=True),
    sa.Column('capex_equipment_misc', sa.FLOAT(), nullable=True),
    sa.Column('capex_facilities', sa.FLOAT(), nullable=True),
    sa.Column('capex_land', sa.FLOAT(), nullable=True),
    sa.Column('capex_per_mw', sa.FLOAT(), nullable=True),
    sa.Column('capex_roads', sa.FLOAT(), nullable=True),
    sa.Column('capex_structures', sa.FLOAT(), nullable=True),
    sa.Column('capex_total', sa.FLOAT(), nullable=True),
    sa.Column('capex_wheels_turbines_generators', sa.FLOAT(), nullable=True),
    sa.Column('construction_type', sa.VARCHAR(length=12), nullable=True),
    sa.Column('construction_year', sa.INTEGER(), nullable=True),
    sa.Column('energy_used_for_pumping_mwh', sa.FLOAT(), nullable=True),
    sa.Column('installation_year', sa.INTEGER(), nullable=True),
    sa.Column('net_generation_mwh', sa.FLOAT(), nullable=True),
    sa.Column('net_load_mwh', sa.FLOAT(), nullable=True),
    sa.Column('opex_dams', sa.FLOAT(), nullable=True),
    sa.Column('opex_electric', sa.FLOAT(), nullable=True),
    sa.Column('opex_engineering', sa.FLOAT(), nullable=True),
    sa.Column('opex_generation_misc', sa.FLOAT(), nullable=True),
    sa.Column('opex_misc_plant', sa.FLOAT(), nullable=True),
    sa.Column('opex_operations', sa.FLOAT(), nullable=True),
    sa.Column('opex_per_mwh', sa.FLOAT(), nullable=True),
    sa.Column('opex_plant', sa.FLOAT(), nullable=True),
    sa.Column('opex_production_before_pumping', sa.FLOAT(), nullable=True),
    sa.Column('opex_pumped_storage', sa.FLOAT(), nullable=True),
    sa.Column('opex_pumping', sa.FLOAT(), nullable=True),
    sa.Column('opex_rents', sa.FLOAT(), nullable=True),
    sa.Column('opex_structures', sa.FLOAT(), nullable=True),
    sa.Column('opex_total', sa.FLOAT(), nullable=True),
    sa.Column('opex_total_nonfuel', sa.FLOAT(), nullable=True),
    sa.Column('opex_water_for_power', sa.FLOAT(), nullable=True),
    sa.Column('peak_demand_mw', sa.FLOAT(), nullable=True),
    sa.Column('plant_capability_mw', sa.FLOAT(), nullable=True),
    sa.Column('plant_hours_connected_while_generating', sa.FLOAT(), nullable=True),
    sa.Column('plant_id_pudl', sa.INTEGER(), nullable=True),
    sa.Column('project_num', sa.INTEGER(), nullable=True),
    sa.ForeignKeyConstraint(['plant_id_pudl'], ['core_pudl__entity_plants_pudl.plant_id_pudl'], name='fk__out_ferc1__yearly_pumped_storage_plants_sched408_plant_id_pudl_core_pudl__entity_plants_pudl'),
    sa.ForeignKeyConstraint(['utility_id_ferc1', 'plant_name_ferc1'], ['core_pudl__assn_ferc1_pudl_plants.utility_id_ferc1', 'core_pudl__assn_ferc1_pudl_plants.plant_name_ferc1'], name='fk__out_ferc1__yearly_pumped_storage_plants_sched408_utility_id_ferc1_core_pudl__assn_ferc1_pudl_plants'),
    sa.ForeignKeyConstraint(['utility_id_pudl'], ['core_pudl__entity_utilities_pudl.utility_id_pudl'], name='fk__out_ferc1__yearly_pumped_storage_plants_sched408_utility_id_pudl_core_pudl__entity_utilities_pudl'),
    sa.PrimaryKeyConstraint('record_id', name='pk__out_ferc1__yearly_pumped_storage_plants_sched408')
    )
    op.drop_table('out_ferc1__yearly_steam_plants_sched402')
    op.drop_table('out_ferc1__yearly_small_plants_sched410')
    op.drop_table('out_ferc1__yearly_pumped_storage_plants_sched408')
    op.drop_table('out_ferc1__yearly_hydroelectric_plants_sched406')
    # ### end Alembic commands ###
